<?php

namespace App\Http\Controllers\Backend\DataManagement;

use Illuminate\Http\Request;
use App\Http\Services\Backend\DataManagement\ActivityManagementService;
use App\Http\Controllers\Controller;
use App\Http\Services\ConstDef\GeneralConst;
use App\Http\Services\CheckParamService;
use Redirect;

class ActivityManagementController extends Controller
{
    // 相關私有服務層調用器宣告
    private $activityManagementSv;
    private $checkParamSv;
    
    public function __construct()
    {
        // 建構服務層調用器
        $this->activityManagementSv = new ActivityManagementService();
        $this->checkParamSv = new CheckParamService();
        // 時區調整
        date_default_timezone_set("Asia/Taipei");
    }
    
    public function index()
    {
        // 渲染
        return view('admin.DataManagement.ActivityManagement');
    }
    
    public function grid()
    {
        // 取得參數
        $param = $_GET;
        if ($param == null) exit();
        $draw = $param["draw"]; //客戶端傳來的查詢次數，無條件回傳用以核對
        $orderColumn = $param["order"][0]["column"] + 1; //前端從 0 開始送，但 mysql 從 1 開始算
        $orderDir = $param["order"][0]["dir"];
        $start = $param["start"];   // 頁碼
        $length = $param["length"]; // 一頁多大
        $searchValue = $param["search"]["value"];
        //客製化搜尋欄位
        // 驗證
        //資料庫
        $recordsTotal = 0;
        $result = $this->activityManagementSv->getActivitys(
            $recordsTotal,
            $orderColumn,
            $orderDir,
            $start,
            $length,
            $searchValue
        );
        // 整理返回資料
        $data = array();
        for ($i = 0; $i < count($result); $i++) {
            $data[] = array(
                //一般資料
                $result[ $i ]["id"],
                $result[ $i ]["activityName"],
                $result[ $i ]["reqTime"],
                $result[ $i ]["drawTime"],
                $result[ $i ]["drawNumbers"],
                $result[ $i ]["drawBtn"],
                $result[ $i ]["redeemTime"],
                $result[ $i ]["checkBtn"],
                $result[ $i ]["checkTime"],
                $result[ $i ]["getTime"],
                $result[ $i ]["getBtn"],
                $result[ $i ]["cdate"],
                $result[ $i ]["mdate"],
                $result[ $i ]["oid"],
            );
        }
        $json = array(
            "draw"            => $draw,
            "recordsTotal"    => $recordsTotal,
            "recordsFiltered" => $recordsTotal, //其實還是填入所有筆數，本次筆數可從陣列取得
            "data"            => $data,
        );
        
        // 返回
        return json_decode(json_encode($json, JSON_NUMERIC_CHECK), true);
    }
    
    public function create()
    {
        // 渲染
        return view('admin.DataManagement.ActivityManagementEdit', [
            'operdata' => "",
        ]);
    }
    
    public function edit($id)
    {
        // 取得參數與驗證
        // 服務層取得資料(以及實現各種業務上的邏輯)
        $activity = $this->activityManagementSv->getActivityById($id);
        
        // 渲染
        return view('admin.DataManagement.ActivityManagementEdit', [
            'operdata' => $activity,
        ]);
    }
    
    public function store(Request $request)
    {
        // 取得參數與驗證
        $mode = $request->mode;
        $id = ($request->mode == 'insert') ? '' : $request->id;
        $activityName = $request->activityName;
        $reqTimeBegin = $request->reqTimeBegin;
        $reqTimeFinal = $request->reqTimeFinal;
        $drawTime = $request->drawTime;
        $drawNumbers = $request->drawNumbers ?? '';
        $redeemTimeBegin = $request->redeemTimeBegin;
        $redeemTimeFinal = $request->redeemTimeFinal;
        $checkTimeBegin = $request->checkTimeBegin;
        $checkTimeFinal = $request->checkTimeFinal;
        $getTimeBegin = $request->getTimeBegin;
        $getTimeFinal = $request->getTimeFinal;
        // 服務層設置(以及實現各種業務上的邏輯)
        if ($mode == "insert") {
            // 新增模式
            $id = $this->activityManagementSv->insertActivity($activityName, $reqTimeBegin, $reqTimeFinal, $drawTime, $drawNumbers, $redeemTimeBegin, $redeemTimeFinal, $checkTimeBegin, $checkTimeFinal, $getTimeBegin, $getTimeFinal, $request->user()->id);
        } else {
            // 編輯模式
            $this->activityManagementSv->modifyActivity($id, $activityName, $reqTimeBegin, $reqTimeFinal, $drawTime, $drawNumbers, $redeemTimeBegin, $redeemTimeFinal, $checkTimeBegin, $checkTimeFinal, $getTimeBegin, $getTimeFinal, $request->user()->id);
        }
        
        // 跳轉
        return redirect('/backend/dataManagement/activityManagement/edit/' . $id);
    }
    
    public function draw($id, Request $request)
    {
        $this->activityManagementSv->draw($id, $request->user()->id);
        
        return redirect('/backend/dataManagement/activityManagement');
    }
    
    public function check($id, Request $request)
    {
        $this->activityManagementSv->check($id, $request->user()->id);
        
        return redirect('/backend/dataManagement/activityManagement');
    }
    
    public function get($id, Request $request)
    {
        $this->activityManagementSv->get($id, $request->user()->id);
        
        return redirect('/backend/dataManagement/activityManagement');
    }
    
}