<?php

namespace App\Http\Controllers\Backend\DataManagement;

use Illuminate\Http\Request;
use App\Http\Services\Backend\DataManagement\GameManagementService;
use App\Http\Services\Backend\DataManagement\GameTmpManagementService;
use App\Http\Services\Backend\DataManagement\SettingManagementService;
use App\Http\Controllers\Controller;
use App\Http\Services\ConstDef\GeneralConst;
use App\Http\Services\CheckParamService;
use Redirect;

class GameManagementController extends Controller
{
    // 相關私有服務層調用器宣告
    private $gameManagementSv;
    private $checkParamSv;
    
    public function __construct()
    {
        // 建構服務層調用器
        $this->gameManagementSv = ((new SettingManagementService())->getSetting()['GAME_AUDIT_SWITCH'] == 'false') ? new GameManagementService() : new GameTmpManagementService();
        $this->checkParamSv = new CheckParamService();
        // 時區調整
        date_default_timezone_set("Asia/Taipei");
    }
    
    public function index()
    {
        // 渲染
        return view('admin.DataManagement.GameManagement');
    }
    
    public function grid()
    {
        // 取得參數
        $param = $_GET;
        if ($param == null) exit();
        $draw = $param["draw"]; //客戶端傳來的查詢次數，無條件回傳用以核對
        $orderColumn = $param["order"][0]["column"] + 1; //前端從 0 開始送，但 mysql 從 1 開始算
        $orderDir = $param["order"][0]["dir"];
        $start = $param["start"];   // 頁碼
        $length = $param["length"]; // 一頁多大
        $searchValue = $param["search"]["value"];
        //客製化搜尋欄位
        // 驗證
        //資料庫
        $recordsTotal = 0;
        $result = $this->gameManagementSv->getGames(
            $recordsTotal,
            $orderColumn,
            $orderDir,
            $start,
            $length,
            $searchValue
        );
        // 整理返回資料
        $data = array();
        for ($i = 0; $i < count($result); $i++) {
            $data[] = array(
                //一般資料
                $result[ $i ]["id"],
                $result[ $i ]["ratio"],
                $result[ $i ]["gp"],
                $result[ $i ]["cdate"],
                $result[ $i ]["mdate"],
                $result[ $i ]["oid"],
            );
        }
        $json = array(
            "draw"            => $draw,
            "recordsTotal"    => $recordsTotal,
            "recordsFiltered" => $recordsTotal, //其實還是填入所有筆數，本次筆數可從陣列取得
            "data"            => $data,
        );
        
        // 返回
        return json_decode(json_encode($json, JSON_NUMERIC_CHECK), true);
    }
    
    public function create()
    {
        // 渲染
        return view('admin.DataManagement.GameManagementEdit', [
            'operdata' => "",
        ]);
    }
    
    public function edit($id)
    {
        // 取得參數與驗證
        // 服務層取得資料(以及實現各種業務上的邏輯)
        $game = $this->gameManagementSv->getGameById($id);
        
        // 渲染
        return view('admin.DataManagement.GameManagementEdit', [
            'operdata' => $game,
        ]);
    }
    
    public function store(Request $request)
    {
        // 取得參數與驗證
        $mode = $request->mode;
        $id = ($request->mode == 'insert') ? '' : $request->id;
        $ratio = $request->ratio;
        $gp = $request->gp;
        // 服務層設置(以及實現各種業務上的邏輯)
        if ($mode == "insert") {
            // 新增模式
            $id = $this->gameManagementSv->insertGame($ratio, $gp, $request->user()->id);
            
        } else {
            // 編輯模式
            $this->gameManagementSv->modifyGame($id, $ratio, $gp, $request->user()->id);
        }
        
        // 跳轉
        return redirect('/backend/dataManagement/gameManagement/edit/' . $id);
    }
    
}