<?php

namespace App\Http\Services\Backend\DataManagement;

use App\Http\Services\ConstDef\GeneralConst;
use App\Models\Web\Player;
use App\Models\Web\Syslog;
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use Illuminate\Support\Facades\DB;

class PlayerManagementService
{
    // 相關私有 model 調用器宣告
    private $playerManagementDb;
    private $syslogManagementDb;
    
    public function __construct()
    {
        // 建構 model 調用器
        $this->playerManagementDb = new Player();
        $this->syslogManagementDb = new Syslog();
    }
    
    public function getPlayers(
        &$cnt = 0,
        $orderColumn,
        $orderDir,
        $start,
        $length,
        $searchValue,
        $lineId,
//        $userName,
        $createDateStart,
        $createDateFinal
    )
    {
        $players = $this->playerManagementDb
            ->select([
                'id',
                'lineId',
                //                'userName',
                //                \DB::raw("CONCAT('<img src=\"', userPhoto, '\" width=\"50\" height=\"50\">') as userPhoto"),
                'gp',
                'cdate',
                'mdate',
            ]);
        // 過濾搜尋條件
        if ($lineId != '') $players = $players->where('lineId', 'like', '%' . $lineId . '%');
//        if ($userName != '') $players = $players->where('userName', 'like', '%' . $userName . '%');
        $players = $players->where('cdate', '>=', $createDateStart . ' 00:00:00');
        $players = $players->where('cdate', '<=', $createDateFinal . ' 23:59:59');
        // 取總筆數
        $cnt = $players->count();
        // 排序
        $players = $players
            ->orderByRaw((int)$orderColumn . ' ' . 'ASC');
        // 彙整
        // 分頁
        $players = $players
            ->skip($start)->take($length);
        // 實際取資料
        $players = $players
            ->get()
            ->toArray();
        
        // 整理返回值並返回
        return $players;
    }
    
    public function getExports($param)
    {
        $players = $this->playerManagementDb
            ->select([
                'id',
                'lineId',
                //                'userName',
                'gp',
                'cdate',
                'mdate',
            ]);
        // 過濾搜尋條件
        if ($param["lineId"] != '') $players = $players->where('lineId', 'like', '%' . $param["lineId"] . '%');
//        if ($param["userName"] != '') $players = $players->where('userName', 'like', '%' . $param["userName"] . '%');
        $players = $players->where('cdate', '>=', $param["createDateStart"] . ' 00:00:00');
        $players = $players->where('cdate', '<=', $param["createDateFinal"] . ' 23:59:59');
        // 實際取資料
        $players = $players
            ->get()
            ->toArray();
        
        // 整理返回值並返回
        return $players;
    }
    
    public function downloadExcel($titles = [], $datas = [], $fileName = 'simple')
    {
        $spreadsheet = new Spreadsheet();
        ini_set('memory_limit', '1024M');
        ini_set("max_execution_time", "600");
        $spreadsheet->getActiveSheet()
            ->fromArray(
                $titles, // The data to set
                null, // Array values with this value will not be set
                'A1' // Top left coordinate of the worksheet range where we want to set these values (default is A1)
            );
        
        $spreadsheet->getActiveSheet()
            ->fromArray(
                $datas, // The data to set
                null, // Array values with this value will not be set
                'A2' // Top left coordinate of the worksheet range where we want to set these values (default is A1)
            );
        
        // Redirect output to a client’s web browser (Xlsx)
        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment;filename="' . $fileName . '.xlsx"');
        header('Cache-Control: max-age=0');
        // If you're serving to IE 9, then the following may be needed
        header('Cache-Control: max-age=1');
        
        // If you're serving to IE over SSL, then the following may be needed
        header('Expires: Mon, 26 Jul 1997 05:00:00 GMT'); // Date in the past
        header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT'); // always modified
        header('Cache-Control: cache, must-revalidate'); // HTTP/1.1
        header('Pragma: public'); // HTTP/1.0
        
        $writer = IOFactory::createWriter($spreadsheet, 'Xlsx');
        $writer->save('php://output');
        exit;
    }
    
    public function gpclear($oid)
    {
        $this->playerManagementDb->where('id', '>=', 0)->update(['gp' => 0]);
        $rc = \DB::select("SELECT ROW_COUNT() AS rc;");
        $rc = $rc[0]->rc;
        // syslog
        $this->syslogManagementDb
            ->insert([
                'type'    => GeneralConst::LOG_ADMIN,
                'func'    => __FUNCTION__,
                'k'       => $oid,
                'memoIn'  => json_encode([], JSON_UNESCAPED_UNICODE),
                'memoOut' => json_encode(['rc' => $rc], JSON_UNESCAPED_UNICODE),
                'cdate'   => date("Y-m-d H:i:s"),
            ]);
        
        return true;
    }
    
}