<?php

namespace App\Http\Services\Backend\DataManagement;

use App\Http\Services\ConstDef\GeneralConst;
use App\Models\Web\Syslogact;
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use Illuminate\Support\Facades\DB;

class SyslogactManagementService
{
    // 相關私有 model 調用器宣告
    private $syslogactManagementDb;
    
    public function __construct()
    {
        // 建構 model 調用器
        $this->syslogactManagementDb = new Syslogact();
    }
    
    public function getSyslogacts(
        &$cnt = 0,
        $orderColumn,
        $orderDir,
        $start,
        $length,
        $searchValue,
        $k,
        $type,
        $createDateStart,
        $createDateFinal
    )
    {
        $syslogacts = $this->syslogactManagementDb
            ->select([
                'id',
                'type',
                'func',
                'k',
                'memoIn',
                'memoOut',
                'cdate',
            ]);
        // 過濾搜尋條件
        if ($k != '') $syslogacts = $syslogacts->where('k', '=', $k);
        if ($type != '') $syslogacts = $syslogacts->where('type', '=', $type);
        $syslogacts = $syslogacts->where('cdate', '>=', $createDateStart . ' 00:00:00');
        $syslogacts = $syslogacts->where('cdate', '<=', $createDateFinal . ' 23:59:59');
        // 取總筆數
        $cnt = $syslogacts->count();
        // 排序
        $syslogacts = $syslogacts
            ->orderByRaw((int)$orderColumn . ' ' . 'DESC');
        // 彙整
        // 分頁
        $syslogacts = $syslogacts
            ->skip($start)->take($length);
        // 實際取資料
        $syslogacts = $syslogacts
            ->get()
            ->toArray();
        
        // 整理返回值並返回
        return $syslogacts;
    }
    
    public function getExports($param)
    {
        $syslogacts = $this->syslogactManagementDb
            ->select([
                'id',
                'type',
                'func',
                'k',
                'memoIn',
                'memoOut',
                'cdate',
            ]);
        // 過濾搜尋條件
        if ($param["k"] != '') $syslogacts = $syslogacts->where('k', 'like', '%' . $param["k"] . '%');
        if ($param["type"] != '') $syslogacts = $syslogacts->where('type', 'like', '%' . $param["type"] . '%');
        $syslogacts = $syslogacts->where('cdate', '>=', $param["createDateStart"] . ' 00:00:00');
        $syslogacts = $syslogacts->where('cdate', '<=', $param["createDateFinal"] . ' 23:59:59');
        // 實際取資料
        $syslogacts = $syslogacts
            ->get()
            ->toArray();
        
        // 整理返回值並返回
        return $syslogacts;
    }
    
    public function downloadExcel($titles = [], $datas = [], $fileName = 'simple')
    {
        $spreadsheet = new Spreadsheet();
        ini_set('memory_limit', '1024M');
        ini_set("max_execution_time", "600");
        $spreadsheet->getActiveSheet()
            ->fromArray(
                $titles, // The data to set
                null, // Array values with this value will not be set
                'A1' // Top left coordinate of the worksheet range where we want to set these values (default is A1)
            );
        
        $spreadsheet->getActiveSheet()
            ->fromArray(
                $datas, // The data to set
                null, // Array values with this value will not be set
                'A2' // Top left coordinate of the worksheet range where we want to set these values (default is A1)
            );
        
        // Redirect output to a client’s web browser (Xlsx)
        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment;filename="' . $fileName . '.xlsx"');
        header('Cache-Control: max-age=0');
        // If you're serving to IE 9, then the following may be needed
        header('Cache-Control: max-age=1');
        
        // If you're serving to IE over SSL, then the following may be needed
        header('Expires: Mon, 26 Jul 1997 05:00:00 GMT'); // Date in the past
        header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT'); // always modified
        header('Cache-Control: cache, must-revalidate'); // HTTP/1.1
        header('Pragma: public'); // HTTP/1.0
        
        $writer = IOFactory::createWriter($spreadsheet, 'Xlsx');
        $writer->save('php://output');
        exit;
    }
    
}