<?php

namespace App\Http\Services\Backend\DataManagement;

use App\Http\Services\ConstDef\GeneralConst;
use App\Models\Web\TSession;
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use Illuminate\Support\Facades\DB;
use App\Models\Web\TRound;

class TSessionManagementService
{
    // 相關私有 model 調用器宣告
    private $tsessionManagementDb;
    private $troundManagementDb;
    
    public function __construct()
    {
        // 建構 model 調用器
        $this->tsessionManagementDb = new TSession();
        $this->troundManagementDb = new TRound();
    }
    
    public function getTRounds()
    {
        $trounds = $this->troundManagementDb->select([
            'id',
            'roundName',
        ])
            ->get()
            ->toArray();
        
        return $trounds;
    }
    
    public function getTSessions(
        &$cnt = 0,
        $orderColumn,
        $orderDir,
        $start,
        $length,
        $searchValue,
        $lineId,
        $userName,
        $tround,
        $createDateStart,
        $createDateFinal
    )
    {
        $tsessions = $this->tsessionManagementDb
            ->leftJoin('trounds', 'tsessions.rid', '=', 'trounds.id')
            ->leftJoin('gameGpRatio', 'tsessions.gid', '=', 'gameGpRatio.id')
            ->select([
                'tsessions.id',
                'tsessions.lineId',
                'tsessions.userName',
                'trounds.roundName',
                'tsessions.gid',
                'tsessions.eventDate',
                'tsessions.currentGameGp',
                'tsessions.currentCheckinGp',
                'tsessions.currentGp',
                'tsessions.cdate',
            ]);
        // 過濾搜尋條件
        if ($lineId != '') $tsessions = $tsessions->where('tsessions.lineId', 'like', '%' . $lineId . '%');
        if ($userName != '') $tsessions = $tsessions->where('tsessions.userName', 'like', '%' . $userName . '%');
        if ($tround != '') $tsessions = $tsessions->where('trounds.id', '=', $tround);
        $tsessions = $tsessions->where('tsessions.cdate', '>=', $createDateStart . ' 00:00:00');
        $tsessions = $tsessions->where('tsessions.cdate', '<=', $createDateFinal . ' 23:59:59');
        // 取總筆數
        $cnt = $tsessions->count();
        // 排序
        $tsessions = $tsessions
            ->orderByRaw((int)$orderColumn . ' ' . 'ASC');
        // 彙整
        // 分頁
        $tsessions = $tsessions
            ->skip($start)->take($length);
        // 實際取資料
        $tsessions = $tsessions
            ->get()
            ->toArray();
        
        // 整理返回值並返回
        return $tsessions;
    }
    
    public function getExports($param)
    {
        $tsessions = $this->tsessionManagementDb
            ->leftJoin('trounds', 'tsessions.rid', '=', 'trounds.id')
            ->select([
                'tsessions.id',
                'tsessions.lineId',
                'tsessions.userName',
                'trounds.roundName',
                'tsessions.gid',
                'tsessions.eventDate',
                'tsessions.currentGameGp',
                'tsessions.currentCheckinGp',
                'tsessions.currentGp',
                'tsessions.cdate',
            ]);
        // 過濾搜尋條件
        if ($param["lineId"] != '') $tsessions = $tsessions->where('tsessions.lineId', 'like', '%' . $param["lineId"] . '%');
        if ($param["userName"] != '') $tsessions = $tsessions->where('tsessions.userName', 'like', '%' . $param["userName"] . '%');
        if ($param["tround"] != '') $tsessions = $tsessions->where('trounds.id', '=', $param["tround"]);
        $tsessions = $tsessions->where('tsessions.cdate', '>=', $param["createDateStart"] . ' 00:00:00');
        $tsessions = $tsessions->where('tsessions.cdate', '<=', $param["createDateFinal"] . ' 23:59:59');
        // 實際取資料
        $tsessions = $tsessions
            ->get()
            ->toArray();
        
        // 整理返回值並返回
        return $tsessions;
    }
    
    public function downloadExcel($titles = [], $datas = [], $fileName = 'simple')
    {
        $spreadsheet = new Spreadsheet();
        ini_set('memory_limit', '1024M');
        ini_set("max_execution_time", "600");
        $spreadsheet->getActiveSheet()
            ->fromArray(
                $titles, // The data to set
                null, // Array values with this value will not be set
                'A1' // Top left coordinate of the worksheet range where we want to set these values (default is A1)
            );
        
        $spreadsheet->getActiveSheet()
            ->fromArray(
                $datas, // The data to set
                null, // Array values with this value will not be set
                'A2' // Top left coordinate of the worksheet range where we want to set these values (default is A1)
            );
        
        // Redirect output to a client’s web browser (Xlsx)
        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment;filename="' . $fileName . '.xlsx"');
        header('Cache-Control: max-age=0');
        // If you're serving to IE 9, then the following may be needed
        header('Cache-Control: max-age=1');
        
        // If you're serving to IE over SSL, then the following may be needed
        header('Expires: Mon, 26 Jul 1997 05:00:00 GMT'); // Date in the past
        header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT'); // always modified
        header('Cache-Control: cache, must-revalidate'); // HTTP/1.1
        header('Pragma: public'); // HTTP/1.0
        
        $writer = IOFactory::createWriter($spreadsheet, 'Xlsx');
        $writer->save('php://output');
        exit;
    }
    
}